﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using NanoMvvm.Threading;

namespace NanoMvvm.Demo.Simple
{
    public class MatchViewModel : DynamicViewModel<Match>
    {
        public MatchViewModel(Match model) : base(model)
        {
            WrapProperty(m => m.MatchId);
            WrapProperty(m => m.Team1Name, m => m.MatchupString); //raise accompanying property changed event
            WrapProperty(m => m.Team2Name, m => m.MatchupString);
            WrapProperty(m => m.MatchupString);
            WrapProperty(m => m.LeagueName);
            WrapProperty(m => m.PlayLocation);
            WrapProperty(m => m.PlayTime);

            GetResults = new CommandViewModel<AsyncCommand<DelegateCommand>>("Get Results",
                                                                             new AsyncCommand<DelegateCommand>(
                                                                                 new DelegateCommand(LoadResults,
                                                                                                     () => !IsLoaded),
                                                                                 ex =>
                                                                                     {
                                                                                         var msg = new SelectionDialogViewModel("Ok");
                                                                                         msg.Header = "Task message dialog";
                                                                                         msg.Message = "Exception(s):\n" +
                                                                                             ex.Aggregate("",
                                                                                                          (s, e) =>
                                                                                                              {
                                                                                                                  var exc = ExceptionHelper
                                                                                                                      .GetInnerMostException(e);
                                                                                                                  return s + exc.GetType() + ": " 
                                                                                                                      + exc.Message + "\n";
                                                                                                              });

                                                                                         msg.Activate();

                                                                                         IsLoaded = false;
                                                                                         IsLoading = false;
                                                                                     }));

            Cancel = new CommandViewModel<DelegateCommand>("Cancel",
                                                           new DelegateCommand(CancelLoad,
                                                                               () => IsLoading &&
                                                                                     !GetResults.Command
                                                                                          .IsCancelRequested));
            GetResults.Command.IsAutomaticRequeryDisabled = true;
            Cancel.Command.IsAutomaticRequeryDisabled = true;
        }

        private bool isLoaded;
        public bool IsLoaded
        {
            get { return isLoaded; }
            private set
            {
                SetProperty(ref isLoaded, value, () => IsLoaded);
            }
        }

        private bool isLoading;
        public bool IsLoading
        {
            get { return isLoading; }
            private set
            {
                SetProperty(ref isLoading, value, () => IsLoading);
            }
        }

        private MatchResult result;
        public MatchResult Result
        {
            get { return result; }
            set { SetProperty(ref result, value, () => Result); }
        }

        private void LoadResults()
        {
            if(IsLoaded || IsLoading) return;

            IsLoading = true;

            RaiseCanExecuteChangedEvents();

            var data = DataRepository.GetMatchResult(WrappedModel.MatchId);

            if (!GetResults.Command.IsCancelRequested)
            {
                if (data == null)
                {
                    Task.Factory.StartNew(() =>
                                              {
                                                  var msg = new SelectionDialogViewModel("Ok");
                                                  msg.Header = "Data message dialog";
                                                  msg.Message = "No results in yet for match \""
                                                      + WrappedModel.MatchupString + "\"!";

                                                  msg.Activate();
                                              }
                                          , CancellationToken.None
                                          , TaskCreationOptions.None,
                                          new SynchronizationContextTaskScheduler(
                                              GetResults.Command.SynchronizationContext));


                }
                else
                {
                    Result = data;

                    IsLoaded = true;
                }
            }

            IsLoading = false;

            RaiseCanExecuteChangedEvents();
        }

        private void RaiseCanExecuteChangedEvents()
        {
            GetResults.Command.RaiseCanExecuteChanged();
            Cancel.Command.RaiseCanExecuteChanged();
        }

        private void CancelLoad()
        {
            GetResults.Command.RequestCancel();

            RaiseCanExecuteChangedEvents();
        }

        private CommandViewModel<AsyncCommand<DelegateCommand>> getResults;
        public CommandViewModel<AsyncCommand<DelegateCommand>> GetResults
        {
            get { return getResults; }
            private set {SetProperty(ref getResults, value, ()=>GetResults); }
        }

        private CommandViewModel<DelegateCommand> cancel;
        public CommandViewModel<DelegateCommand> Cancel
        {
            get { return cancel; }
            private set { SetProperty(ref cancel, value, () => Cancel); }
        }
    }
}