﻿using System.Diagnostics;
using NanoMvvm;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

namespace NanoMvvm.UnitTests
{
    [TestClass]
    public class WeakActionContainerTest
    {
        private class TestClass
        {
            public void TestActionMethod(int p)
            {
                Debug.WriteLine("TestActionMethod executed.");
            }

            public void TestGenericActionMethod<T>(T p)
            {
                Debug.WriteLine("TestGenericActionMethod executed.");
            }

            public static void StaticTestActionMethod(int p)
            {
                Debug.WriteLine("StaticTestActionMethod executed.");
            }
        }

        [TestMethod]
        public void WeakActionContainerInstanceMethodTest()
        {
            var a = new TestClass();

            var c = new WeakActionContainer<int>(a.TestActionMethod);

            Assert.IsTrue(c.IsTargetAlive);

            c.Execute(10);

            a = null;

            GC.Collect();

            Assert.IsFalse(c.IsTargetAlive);
        }

        [TestMethod]
        public void WeakActionContainerGenericInstanceMethodTest()
        {
            var a = new TestClass();

            var c = new WeakActionContainer<int>(a.TestGenericActionMethod<int>);

            Assert.IsTrue(c.IsTargetAlive);

            c.Execute(10);

            a = null;

            GC.Collect();

            Assert.IsFalse(c.IsTargetAlive);
        }

        [TestMethod]
        public void WeakActionContainerStaticMethodTest()
        {
            var c = new WeakActionContainer<int>(TestClass.StaticTestActionMethod);

            Assert.IsTrue(c.IsTargetAlive);

            c.Execute(10);
        }

        [TestMethod]
        public void WeakActionContainerInstanceEqualityTest()
        {
            var a = new TestClass();

            var c = new WeakActionContainer<int>(a.TestActionMethod);

            Assert.IsTrue(c.Equals(a.TestActionMethod));
        }

        [TestMethod]
        public void WeakActionContainerStaticEqualityTest()
        {
            var c = new WeakActionContainer<int>(TestClass.StaticTestActionMethod);

            Assert.IsTrue(c.Equals(TestClass.StaticTestActionMethod));
        }

        [TestMethod]
        public void WeakActionContainerAnonymousMethodTest()
        {
            int i = 0;
            i++;

            Action<int> a = (o) => 
            { 
                Debug.WriteLine(o);
                Debug.WriteLine(i);
            };

            var c = new WeakActionContainer<int>(a);

            c.Execute(100);
        }
    }
}
