﻿using System.Diagnostics;
using NanoMvvm;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

namespace NanoMvvm.UnitTests
{
    [TestClass]
    public class WeakFuncContainerTest
    {
        private class TestClass
        {
            public int TestActionMethod(int p)
            {
                Debug.WriteLine("TestActionMethod executed.");
                return p;
            }

            public int TestGenericActionMethod<T>(T p)
            {
                Debug.WriteLine("TestGenericActionMethod executed.");
                return 10;
            }

            public static int StaticTestActionMethod(int p)
            {
                Debug.WriteLine("StaticTestActionMethod executed.");
                return p;
            }
        }

        [TestMethod]
        public void WeakFuncContainerInstanceMethodTest()
        {
            var a = new TestClass();

            var c = new WeakFuncContainer<int, int>(a.TestActionMethod);

            Assert.IsTrue(c.IsTargetAlive);

            Assert.AreEqual(c.Compute(10), 10);

            a = null;

            GC.Collect();

            Assert.IsFalse(c.IsTargetAlive);
        }

        [TestMethod]
        public void WeakFuncContainerGenericInstanceMethodTest()
        {
            var a = new TestClass();

            var c = new WeakFuncContainer<int, int>(a.TestGenericActionMethod<int>);

            Assert.IsTrue(c.IsTargetAlive);

            Assert.AreEqual(c.Compute(20), 10);

            a = null;

            GC.Collect();

            Assert.IsFalse(c.IsTargetAlive);
        }

        [TestMethod]
        public void WeakFuncContainerStaticMethodTest()
        {
            var c = new WeakFuncContainer<int, int>(TestClass.StaticTestActionMethod);

            Assert.IsTrue(c.IsTargetAlive);

            Assert.AreEqual(c.Compute(10), 10);
        }

        [TestMethod]
        public void WeakFuncContainerInstanceEqualityTest()
        {
            var a = new TestClass();

            var c = new WeakFuncContainer<int, int>(a.TestActionMethod);

            Assert.IsTrue(c.Equals(a.TestActionMethod));
        }

        [TestMethod]
        public void WeakFuncContainerStaticEqualityTest()
        {
            var c = new WeakFuncContainer<int, int>(TestClass.StaticTestActionMethod);

            Assert.IsTrue(c.Equals(TestClass.StaticTestActionMethod));
        }

        [TestMethod]
        public void WeakFuncContainerAnonymousMethodTest()
        {
            Func<int, int> f = (o) => o;

            var c = new WeakFuncContainer<int, int>(f);

            Assert.AreEqual(c.Compute(123), 123);
        }
    }
}
